// Copyright 2018 Espressif Systems (Shanghai) PTE LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef _OSAL_H_
#define _OSAL_H_

#include <freertos/FreeRTOS.h>
#include <freertos/task.h>
#include <unistd.h>
#include <stdint.h>
#include <esp_timer.h>

#ifdef __cplusplus
extern "C" {
#endif

#define OS_SUCCESS ESP_OK
#define OS_FAIL    ESP_FAIL
#define HTTPD_OS_THREAD_CORE_ID (0)

typedef TaskHandle_t othread_t;
static StackType_t *httpd_os_thread_task_stack;

static inline int httpd_os_thread_create(othread_t *thread,
                                 const char *name, uint16_t stacksize, int prio,
                                 void (*thread_routine)(void *arg), void *arg)
{
#if CONFIG_SPIRAM_BOOT_INIT
    httpd_os_thread_task_stack = (StackType_t *)heap_caps_malloc(1 * stacksize, MALLOC_CAP_SPIRAM | MALLOC_CAP_8BIT);
    if (httpd_os_thread_task_stack) {
        memset(httpd_os_thread_task_stack, 0, 1 * stacksize);
    } else {
        return OS_FAIL;
    }

    /*
    * Note: Make sure selected the `CONFIG_SPIRAM_BOOT_INIT` and `CONFIG_SPIRAM_ALLOW_STACK_EXTERNAL_MEMORY` by `make menuconfig`
    */
    TaskParameters_t xRegParameters = {
        .pvTaskCode = thread_routine,
        .pcName = name,
        .usStackDepth = stacksize,
        .pvParameters = arg,
        .uxPriority = prio | portPRIVILEGE_BIT,
        .puxStackBuffer = httpd_os_thread_task_stack,
        .xRegions = {{
                .pvBaseAddress = 0x00,
                .ulLengthInBytes = 0x00,
                .ulParameters = 0x00,
            }
        }
    };

    if (xTaskCreateRestrictedPinnedToCore(&xRegParameters, (xTaskHandle)thread, HTTPD_OS_THREAD_CORE_ID) != pdPASS) {
        return OS_FAIL;
    }
#else
    int ret = xTaskCreate(thread_routine, name, stacksize, arg, prio, thread);
    if (ret != pdPASS) {
        return OS_FAIL;
    }
#endif

    return OS_SUCCESS;
}

/* Only self delete is supported */
static inline void httpd_os_thread_delete()
{
    vTaskDelete(xTaskGetCurrentTaskHandle());
    if (httpd_os_thread_task_stack) {
        free(httpd_os_thread_task_stack);
        httpd_os_thread_task_stack = NULL;
    }
}

static inline void httpd_os_thread_sleep(int msecs)
{
    vTaskDelay(msecs / portTICK_RATE_MS);
}

static inline othread_t httpd_os_thread_handle()
{
    return xTaskGetCurrentTaskHandle();
}

#ifdef __cplusplus
}
#endif

#endif /* ! _OSAL_H_ */
